package rescuecore.map09;

import java.util.Random;
import java.lang.Math;
import java.util.Vector;
import java.util.Set;
import java.util.List;
import java.util.HashSet;

import rescuecore.RescueObject;
import rescuecore.Memory;
import traffic.object.MotionlessObject;
import rescuecore.*;
import rescuecore.map09.TestCenter;
import rescuecore.CenterAgent;



public class SampleExplorationAgent extends BaseExplorationAgent {
	Random rand;
	ExplorationTarget currentTarget;
	TestCenter currentCenter;

	Set<Integer> visitedTargets = new HashSet<Integer>();
	
	boolean visitedTargetsIsEmpty = true;
	
	public static final byte[] intToByteArray(int value) 
	{
		             return new byte[] {
		                    (byte)(value >> 24),
		                    (byte)(value >> 16),
		                    (byte)(value >> 8),
		                    (byte)value};
	}
	public static final int byteArrayToInt(byte [] b) 
	{
		             return (b[0] << 24)
		                     | ((b[1] & 0xFF) << 16)
		                     | ((b[2] & 0xFF) << 8)
		                     | (b[3] & 0xFF);
	}
	
	
	public SampleExplorationAgent(String[] args) {
		super(args);
		rand = new Random();
				
		currentTarget = null;
	}

	private boolean isCenter(int id) {
		RescueObject target = memory.lookup(id);
		if (target.isFireStation() || target.isAmbulanceCenter() || target.isPoliceOffice()) return true; // Centers are fire stations, police offices and ambulance centers
		return false;
	}

	protected boolean willListenHear(int from) {
		if(isCenter(from))
			return true;
		return false;
	}

	protected boolean hear(int from, byte[] msg)
	{
		int newVisited  = byteArrayToInt(msg);
		
		System.out.println(id + " Got " + newVisited + " from " + from);
		
		visitedTargets.add(newVisited);
		
		System.out.println(id + "New Visited Targets Contents: " + visitedTargets.toString());
	
		return true;
	}

	protected void sense() 
	{
		if (currentTarget != null) {
			if (getLocation() == currentTarget.target) {
				//System.out.println("Reached " + currentTarget.target.toString());
				currentTarget.explored = true;
				
				byte[] message = intToByteArray(currentTarget.target.getID());
				this.tell(1, message);		
				currentTarget = null;
			}
		}

		if(currentTarget == null) {
			chooseTarget();
		}
	
		if(currentTarget != null)
			moveTo(currentTarget);
		
		byte [] chans = new byte[1];
		chans[0] = 2;
		channel(chans);
	}
	
	private void chooseTarget()
	{
		double sm_distance = 999999.0;
		for(int i=0; i<explorationTargets.size(); i++)
		{
			ExplorationTarget expl = explorationTargets.get(i);			
		
			if(visitedTargets.contains(expl.target.getID())) continue;
			
			double distance = 999999.0;
			try
			{
				distance = memory.getDistance(expl.target, me());
				System.out.println("Dist from me " + id + " to " + expl.target.getID() + " is " + distance);
			}catch(CannotFindLocationException e){}
			
			
			if(distance < sm_distance && !expl.explored)
			{
					sm_distance = distance;
					currentTarget = expl;				
			}
		}
	}
}
